unit IDSetU;

{$A-}  {Pack records here}

interface

uses StdCtrls, ExtCtrls, Graphics, Forms, Buttons, Classes,
   Controls, Windows, Dialogs, SysUtils;

type
  TLongintArray = array [0..16380] of longint;
  PLongArray = ^TLongintArray;

  TIDSet = class (TObject)
  private
    FCapacity:  word;
    FCount:     word;
    FSet:       PLongArray;
  protected
    procedure Grow;
    procedure SetCount(NewCount: word);
    procedure SetCapacity(NewCapacity: word);
    procedure Glue(ri: longint);
  public
    destructor Destroy; override;
    procedure Empty;
    function  Get(w: word): longint;
    function  IsIn(ri: longint; var place: word): boolean;
    procedure Add(ri: longint);
    procedure Delete(ri: longint);
    procedure Append(FromSet: TIDSet);
    procedure Intersection(Set1,Set2: TIDSet);
    procedure Union(Set1,Set2: TIDSet);
    procedure Difference(Set1,Set2: TIDSet);
    procedure Exclusive(Set1,Set2: TIDSet);
    function  SetPointer: pointer;
    property  Capacity: word
       read FCapacity
       write SetCapacity;
    property Count: word
       read FCount
       write SetCount;
    end;

{ Property descriptions

Count:  The number of id's in the IDSet.

Capacity: The number of id's the IDSet CAN hold. }

{ Method descriptions

Empty:  Empties a TIDSet.

IsIn: Returns TRUE if the given longint ri is on the set. In this
case the parameter is set to the place where ri is stored.

Get: Returns the longint stored in slot w of the TIDSet.

Add: Adds ri to the set if not already in the set. If needed the
capacity will be increased to make room for the new item.

Delete: Removes ri from the set, if it is in the set.

Append: Copy any id's from the FromSet to this one if they
are not already present..

Union: Forms the union of Set1 and Set2. The IDSet will contain
an id if it is in Set1 OR in Set 2 or both. (It is included only
once,however.)

Intersection: Forms the intersection of Set1 and Set2. The IDSet
will contain an id if it is in Set1 AND in Set2 also.

Difference: Forms the difference of Set1 and Set2. The IDSet
will contain an id if it is in Set1 but NOT in Set2.

Exclusive: Forms the symmetric difference of Set1 and Set2. The
will contain an id if it is Set1 or Set2 BUT NOT BOTH.

SetPointer. This is intended for use by VFile. }


implementation

{ TIDSet Methods  }

destructor TIDSet.Destroy;

begin
  Empty;
  end;


procedure TIDSet.Empty;

begin
  SetCapacity(0);
  SetCount(0);
  end;


procedure TIDSet.SetCapacity(NewCapacity: word);

var
  PLA: PLongArray;

begin
  if NewCapacity <> FCapacity
  then begin
    if NewCapacity = 0
    then PLA := nil
    else begin
      GetMem(PLA,4*NewCapacity);
      if FCount <> 0
      then Move(FSet^,PLA^,4*FCount);
      end;
    if FCapacity <> 0
    then FreeMem(FSet,4*FCapacity);
    FSet := PLA;
    FCapacity := NewCapacity;
    end;
  end;



procedure TIDSet.SetCount(NewCount: word);

begin
  if NewCount > FCapacity
  then SetCapacity(NewCount);
  FCount := NewCount;
  end;


procedure TIDSet.Grow;

begin
  SetCapacity(FCapacity + 16);
  end;


function  TIDSet.IsIn(ri: longint; var place: word): boolean;

begin
  place := 0;
  if FCount = FCapacity
  then Grow;
  FSet^[FCount] := ri;  {sentinal at end}
  Result := FALSE;
  while not Result do
    if ri = FSet^[place]
    then begin
      Result := place < FCount;
      exit;
      end
    else inc(place);
  Result := place < FCount;
  end;


procedure  TIDSet.Add(ri: longint);

var
  place: word;

begin
  if FCapacity = FCount
  then Grow;
  if IsIn(ri,place)
  then exit;
  FSet^[FCount] := ri;
  inc(FCount);
  end;



procedure  TIDSet.Glue(ri: longint);

begin {This is used if it known that ri is not on the list.}
  if FCapacity = FCount
  then Grow;
  FSet^[FCount] := ri;
  inc(FCount);
  end;



procedure  TIDSet.Delete(ri: longint);

var
  place: word;

begin
  if not IsIn(ri,place)
  then exit;
  dec(FCount);
  FSet^[place] := FSet^[FCount];
  end;


procedure  TIDSet.Append(FromSet: TIDSet);

var
  w: word;

begin
  SetCapacity(FromSet.FCount+FCapacity);
  for w := 1 to FromSet.Count do
    Add(FromSet.Get(w-1))
  end;


procedure  TIDSet.Intersection(Set1,Set2: TIDSet);

var
  w,place: word;

begin
  Empty;
  SetCapacity(Set1.FCount);
  for w := 1 to Set1.FCount do
    if Set2.IsIn(Set1.FSet^[w-1],place)
    then Glue(Set1.Fset^[w-1]);
  end;



procedure  TIDSet.Union(Set1,Set2: TIDSet);

var
  w: word;

begin
  Empty;
  SetCapacity(Set1.FCount+Set2.FCount);
  for w := 1 to Set1.FCount do
    Glue(Set1.FSet^[w-1]);
  for w := 1 to Set2.FCount do
    Glue(Set2.FSet^[w-1]);
  end;



procedure  TIDSet.Difference(Set1,Set2: TIDSet);

var
  w,place: word;

begin
  Empty;
  SetCapacity(Set1.FCount);
  for w := 1 to Set1.FCount do
    if not Set2.IsIn(Set1.Fset^[w-1],place)
    then Glue(Set1.FSet^[w-1]);
  end;


procedure  TIDSet.Exclusive(Set1,Set2: TIDSet);

var
  w,place: word;

begin
  Empty;
  SetCapacity(Set1.Count+Set2.Count);
  for w := 1 to Set1.Count do
    if not Set2.IsIn(Set1.FSet^[w-1],place)
    then Glue(Set1.FSet^[w-1]);
  for w := 1 to Set2.Count do
    if not Set1.IsIn(Set2.FSet^[w-1],place)
    then Glue(Set2.FSet^[w-1]);
  end;


function TIDSet.Get(w: word): longint;

begin
  if w >= FCount
  then Result := -1
  else Result := FSet^[w];
  end;


function TIDSet.SetPointer: pointer;

begin
  Result := FSet;
  end;



  end.


