unit SelectU;

{$A-}  {pack records here}

{====================== SelectU MODULE ==========================

  This module contains the routines that handle the user
  interfaces for descendents of TPKey.


===============================================================}

interface

uses StdCtrls, ExtCtrls, Graphics, Forms, Buttons, Classes,
   Controls, Windows, Dialogs, SysUtils,
   KeyU;

{$A-}  {Make sure these records are packed.}

type
  TSelKey = class (TComponent)
  private
    FCharCase: TEditCharCase;
    FContrast: TColor;
  public
  protected
  private
  {Methods that handle user searches}
  public
    constructor Create(AOwner: TComponent); override;
    function SelectOneKey(Key: TKey; Title: string;
                var found: shortstring): boolean;
    function SelectManyKeys(Key: TKey; Title: string;
                var KSL: TStringList): boolean;
    function SelectOneKeyInsert(Key: TKey; Title: string;
                var ISL: TStringList; var found: shortstring): boolean;
    function SelectManyKeysInsert(Key: TKey; Title: string;
                var ISL: TStringList; var KSL: TStringList): boolean;
  published
  property CharCase: TEditCharCase
      read  FCharCase
      write FCharCase default ecNormal;
    property Contrast:  TColor
      read  FContrast
      write FContrast default clSilver;
   end;

{A description of the methods follows.}

{SelectOneKey:
 TheKey is the TPKey to be used as choices for selection.
 Title will appear as the caption for the Selection Form. found is
 the key selected by the user. TRUE is returned if the user ended
 the session by pressing the OK key. FALSE is returned if the user
 ended by pressing the Cancel key or the close box.

 SelectOneKeyInsert:
 TheKey is the TPKey to be used as choices for selection.
 Title will appear as the caption for the Selection Form. ISL is
 the list of keys the user inserted during the session. You need
 to handle this list after the user finishes. found is the key
 selected by the user. TRUE is returned if the user ended the
 session by pressing the OK key. FALSE is returned if the user
 ended by pressing the Cancel key or the close box.

 SelectManyKeys:
 TheKey is the TPKey to be used as choices for selection.
 Title will appear as the caption for the Selection Form. KSL is
 the list of strings selected by the user.  You need to handle
 this list after the user finishes. found is the key selected by
 the user. TRUE is returned if the user ended the session by
 pressing the OK key. FALSE is returned if the user ended by
 pressing the Cancel key or the close box.

 SelectManyKeysInsert:
 TheKey is the TPKey to be used as choices for selection.
 Title will appear as the caption for the Selection Form. ISL is
 the list of keys the user inserted during the session. You need
 to handle this list after the user finishes. KSL is
 the list of strings selected by the user.  You need to handle
 this list after the user finishes. found is the key selected by
 the user. TRUE is returned if the user ended the session by
 pressing the OK key. FALSE is returned if the user ended by
 pressing the Cancel key or the close box. }


 procedure Register;


implementation

{$R SELECTU.RES}

uses checklst;

type
  TSelectSetUp = record
    TheKey: TKey;
    OneOnly: boolean;
    AllowInserts: boolean;
    Header: string;
    Chosen: string;
    KeyList: TStringList;
    NewList: TStringList;
    TheCharCase: TEditCharCase;
    TheContrast: TColor;
    end;


procedure Register;
begin
  RegisterComponents('Len''s',[TSelKey]);
  end;


{This part defines the SelectForm interface.}
type
  TSelectForm = class(TForm)
  private
    FSetUp: TSelectSetUp;
    {Visual controls}
    pnlFrame: TPanel;
    sbnOK: TSpeedButton;
    sbnCancel: TSpeedButton;
    sbnSync: TSpeedButton;
    sbnInsert: TSpeedButton;
    edtPartial: TEdit;
    clbChoose: TCheckListBox;
    lbxShow: TListBox;
    scbFake: TScrollBar;   {Makes the user think clbChoose is a list box}
  {Local variables}
    ShowScroll: boolean;   {True if more keys than Actual}
    Actual: byte;          {Actual number of keys in clbChoose}
    EditHeight: word;      {Height of edtPArtial}
    IHeight: word;         {Height of list box item}
    Dragger: string;       {Used for dragging keys}
    CheckString: string;
    MinimumHeight: word;

    constructor Init(var SetUp: TSelectSetUp);
    procedure OkClick(Sender: TObject);
    procedure CancelClick(Sender: TObject);
    procedure SyncClick(Sender: TObject);
    procedure InsertClick(Sender: TObject);

    procedure ReadSelections;
    procedure SetThumb(at: shortstring);  {of FakeScrollBar}
    procedure HandlePartialKey;
    procedure scbFakeScroll(Sender: TObject; ScrollCode: TScrollCode;
         var ScrollPos: integer);
    procedure edtPartialChange(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: word;
         Shift: TShiftState);
    procedure FormKeyDown(Sender: TObject; var Key: word;
         Shift: TShiftState);
    procedure TabPress;
    {Routines that handle the clbChoose items}
    procedure Top;
    procedure Bottom;
    procedure Up1;
    procedure Down1;
    procedure PageUp;
    procedure PageDown;
    procedure GoToThumbPosition(ratio: real);
    procedure GoToKey(pk: string);
    procedure SurroundThisKey(s: string);
    {Routines that handle the List Boxes}
    procedure PutInChecks;
    procedure CalculateActual;
    procedure clbChooseClickCheck(Sender: TObject);
    procedure clbChooseDblClick(Sender: TObject);
    procedure clbChooseMouseDown(Sender: TObject; Button: TMouseButton;
                 Shift: TShiftState;  X,Y: integer);
    procedure clbChooseDragOver(Sender, Source: TObject; X,Y: integer;
                 State: TDragState; var Accept: boolean);
    procedure clbChooseDragDrop(Sender, Source: TObject; X,Y: integer);
    procedure lbxShowDblClick(Sender: TObject);
    procedure lbxShowMouseDown(Sender: TObject; Button: TMouseButton;
                 Shift: TShiftState; X,Y: integer);
    procedure lbxShowDragOver(Sender, Source: TObject; X,Y: integer;
                 State: TDragState; var Accept: boolean);
    procedure lbxShowDragDrop(Sender, Source: TObject; X,Y: integer);
    procedure ResizeComponents(Sender: TObject);
    end;


{==========================================================}
{    User interface methods                                }
{==========================================================}

constructor TSelKey.Create(AOwner: TComponent);

begin
  inherited Create(AOwner);
  CharCase := ecNormal;
  Contrast := clSilver;
  end;



{=============================================================}
{           Search methods                                    }
{ Most of the work is absorbed by the Init method. These just }
{set up parameters.                                           }
{=============================================================}

function TSelKey.SelectOneKey(Key: TKey; Title: string;
           var found: shortstring): boolean;

var
  SetUp: TSelectSetUp;
  Select: TSelectForm;
  LocalList: TStringList;

begin
  with SetUp do begin
    TheKey := Key;
    OneOnly := TRUE;
    AllowInserts := FALSE;
    Header := Title;
    Chosen := found;
    LocalList := TStringList.Create;
    KeyList := LocalList;
    NewList := nil;
    TheCharCase := FCharCase;
    TheContrast := FContrast;
    end;
  Select := TSelectForm.Init(SetUp);
  Result := Select.ShowModal = mrOK;
  found := Select.FSetUp.Chosen;
  LocalList.Free;
  Select.Free;
  end;



function TSelKey.SelectManyKeys(Key: TKey; Title: string;
            var KSL: TStringList): boolean;


var
  SetUp: TSelectSetUp;
  Select: TSelectForm;

begin
  with SetUp do begin
    TheKey := Key;
    OneOnly := FALSE;
    AllowInserts := FALSE;
    Header := Title;
    Chosen := '';
    KeyList := KSL;
    NewList := nil;
    TheCharCase := FCharCase;
    TheContrast := FContrast;
    end;
  Select := TSelectForm.Init(SetUp);
  Result := Select.ShowModal = mrOK;
  Select.Free;
  end;



function TSelKey.SelectOneKeyInsert(Key: TKey; Title: string;
          var ISL: TStringList; var found: shortstring): boolean;

var
  SetUp: TSelectSetUp;
  Select: TSelectForm;
  LocalList: TStringList;


begin
  with SetUp do begin
    TheKey := Key;
    OneOnly := TRUE;
    AllowInserts := TRUE;
    Header := Title;
    Chosen := found;
    LocalList := TStringList.Create;
    KeyList := LocalList;
    NewList := ISL;
    TheCharCase := FCharCase;
    TheContrast := FContrast;
    end;
  ISL.Duplicates := dupIgnore;
  Select := TSelectForm.Init(SetUp);
  Result := Select.ShowModal = mrOK;
  found := Select.FSetUp.Chosen;
  LocalList.Free;
  Select.Free;
  end;


function TSelKey.SelectManyKeysInsert(Key: TKey; Title: string;
           var ISL: TStringList; var KSL: TStringList): boolean;

var
  SetUp: TSelectSetUp;
  Select: TSelectForm;

begin
  with SetUp do begin
    TheKey := Key;
    OneOnly := FALSE;
    AllowInserts := TRUE;
    Header := Title;
    Chosen := '';
    KeyList := KSL;
    NewList := ISL;
    TheCharCase := FCharCase;
    TheContrast := FContrast;
    end;
  ISL.Duplicates := dupIgnore;
  Select := TSelectForm.Init(SetUp);
  Result := Select.ShowModal = mrOK;
  Select.Free;
  end;



{============================================================}
{          Methods of TSelectForm                            }
{============================================================}
procedure TSelectForm.Top;

var
  f: shortstring;
  w: word;

begin
  with FSetUp.TheKey do begin
    f := FirstKey;
    clbChoose.Items.Clear;
    for w := 1 to Actual do begin
      clbChoose.Items.Add(f);
      f := NextKey(f);
      end;
    scbFake.Position := scbFake.Min;
    PutInChecks;
    ActiveControl := edtPartial;
    end;
  end;


procedure TSelectForm.Bottom;

var
  f: shortstring;
  w: word;

begin
  with FSetUp.TheKey do begin
    f := LastKey;
    clbChoose.Items.Clear;
    for w := 1 to Actual do begin
      clbChoose.Items.Add(f);
      f := PrevKey(f);
      end;
    scbFake.Position := scbFake.Max;
    PutInChecks;
    ActiveControl := edtPartial;
    end;
  end;


procedure TSelectForm.Up1;

var
  f: shortstring;

begin
  with FSetUp.TheKey do begin
    f := PrevKey(clbChoose.Items[0]);
    if f = ''
    then exit;
    clbChoose.Items.Delete(Actual - 1);
    clbChoose.Items.Add(f);
    scbFake.Position := scbFake.Position -
         scbFake.SmallChange;
    PutInChecks;
    ActiveControl := edtPartial;
    end;
  end;


procedure TSelectForm.Down1;

var
  f: shortstring;

begin
  with FSetUp.TheKey do begin
    f := NextKey(clbChoose.Items[Actual - 1]);
    if f = ''
    then exit;
    clbChoose.Items.Delete(0);
    clbChoose.Items.Add(f);
    scbFake.Position := scbFake.Position +
         scbFake.SmallChange;
    PutInChecks;
    ActiveControl := edtPartial;
    end;
  end;


procedure TSelectForm.PageUp;

var
  f: shortstring;
  w: word;

begin
  with FSetUp.TheKey do begin
    f := PrevKey(clbChoose.Items[0]);
    w := 0;
    while (w < Actual) and (f <> '') do begin
      clbChoose.Items.Delete(Actual - 1);
      clbChoose.Items.Add(f);
      inc(w);
      f := PrevKey(f);
      end;
    scbFake.Position := scbFake.Position -
         scbFake.LargeChange;
    PutInChecks;
    ActiveControl := edtPartial;
    end;
  end;


procedure TSelectForm.PageDown;

var
  f: shortstring;
  w: word;

begin
  with FSetUp.TheKey do begin
    f := NextKey(clbChoose.Items[Actual - 1]);
    w := 0;
    while (w < Actual) and (f <> '') do begin
      clbChoose.Items.Delete(0);
      clbChoose.Items.Add(f);
      inc(w);
      f := NextKey(f);
      end;
    scbFake.Position := scbFake.Position +
         scbFake.LargeChange;
    PutInChecks;
    ActiveControl := edtPartial;
    end;
  end;


procedure TSelectForm.GoToThumbPosition(ratio: real);

var
  f: shortstring;

begin
  if FSetUp.TheKey.ActiveKeys > 0
  then begin
    FSetUp.TheKey.KeyByRatio(ratio,f);
    SurroundThisKey(f);
    SetThumb(clbChoose.Items[Actual div 2]);
    PutInChecks;
    ActiveControl := edtPartial;
    end;
  end;


procedure TSelectForm.GoToKey(pk: string);

var
  f: shortstring;

begin
  if pk = ''
  then begin
    f := FSetUp.TheKey.FirstKey;
    SurroundThisKey(f);
    clbChoose.ItemIndex := -1
    end
  else begin
    f := FSetUp.TheKey.FindKey(pk);
    if f = ''
    then f := FSetUp.TheKey.LastKey;
    SurroundThisKey(f);
    clbChoose.ItemIndex := clbChoose.Items.IndexOf(f);
    end;
  SetThumb(clbChoose.Items[Actual div 2]);
  PutInChecks;
  ActiveControl := edtPartial;
  end;



procedure TSelectForm.SurroundThisKey(s: string);
{This routine insures that the key s is somewhere in the  list box.
It also prevents there being only the last key shown when the end of the
key list is reached.}

var
  f: shortstring;
  u,d: word;

begin
  with FSetUp.TheKey do begin
    clbChoose.Items.Clear;
    {First go up}
    f := FindKey(s);
    u := 0;
    while (u < Actual) and (f <> '') do begin
      clbChoose.Items.Add(f);
      inc(u);
      f := PrevKey(f);
      end;
    {Now go down}
    f := FindKey(s);
    d := 0; f := NextKey(f);
    while (d < Actual) and (f <> '') do begin
      clbChoose.Items.Add(f);
      inc(d);
      f := NextKey(f);
      end;
    {Now delete any excess from above and below the center, if possible}
    while (u + d > Actual) do
      if d > u
      then begin
        clbChoose.Items.Delete(clbChoose.Items.Count - 1);
        dec(d);
        end
      else begin
        clbChoose.Items.Delete(0);
        dec(u);
        end;
    end;
  end;


procedure TSelectForm.SetThumb(at: shortstring);

var
  ratio: real;

begin
  if FSetUp.TheKey.ActiveKeys > 0
  then begin
    ratio := FSetUp.TheKey.FindRatio(at);
    scbFake.Position := Round(ratio * scbFake.Max);
    end;
  end;


procedure TSelectForm.clbChooseClickCheck(Sender: TObject);

begin
  with clbChoose do begin
    if ItemIndex = -1
    then exit;
    CheckString := Items[ItemIndex];
    if Checked[ItemIndex]
    then if FSetUp.OneOnly
         then begin
           lbxShow.Clear;
           lbxShow.Items.Add(CheckString)
           end
         else if lbxShow.Items.IndexOf(CheckString) = -1
              then lbxShow.Items.Add(CheckString)
              else
    else if lbxShow.Items.IndexOf(CheckString) <> -1
         then lbxShow.Items.Delete(
           lbxShow.Items.IndexOf(CheckString))
         else;
    end;
    PutInChecks;
  end;


procedure TSelectForm.clbChooseDblClick(Sender: TObject);

begin
  with clbChoose do begin
    if ItemIndex = -1
    then exit;
    CheckString := Items[ItemIndex];
    if FSetUp.OneOnly
    then begin
      lbxShow.Clear;
      lbxShow.Items.Add(CheckString);
      end
    else if lbxShow.Items.IndexOf(CheckString) = -1
         then lbxShow.Items.Add(CheckString);
    PutInChecks;
    end;
  end;



procedure TSelectForm.lbxShowDblClick(Sender: TObject);

var
  j: integer;

begin
  j := lbxShow.ItemIndex;
  if j = -1
  then exit;
  lbxShow.Items.Delete(j);
  lbxShow.ItemIndex := -1;  {Turn off highlight}
  PutInChecks;
  ActiveControl := edtPartial;
end;


procedure TSelectForm.PutInChecks;

var
  Index: integer;
  w: word;

begin
  for w := 1 to Actual do begin
    Index := lbxShow.Items.IndexOf(clbChoose.Items[w-1]);
    if Index >= 0
    then clbChoose.Checked[w-1] := TRUE
    else clbChoose.Checked[w-1] := FALSE;
    end;
  lbxShow.ItemIndex := -1;  {Turn off highlight in lbxShow}
  end;




{=========================================================}
{  Part 2   Auxiliary routines                            }
{=========================================================}

procedure TSelectForm.ReadSelections;

var
  i: word;

begin
  with FSetUp do begin
    if OneOnly
    then Chosen := lbxShow.Items[0]
    else begin
      KeyList.Clear;
      for i := 1 to lbxShow.Items.Count do
        KeyList.Add(lbxShow.Items[i-1]);
      end;
    end;
  end;


procedure TSelectForm.HandlePartialKey;

begin
  if FSetUp.TheKey.ActiveKeys > 0
  then begin
    GoToKey(edtPartial.Text);
    ActiveControl := edtPartial;
    end;
  end;



procedure TSelectForm.edtPartialChange(Sender: TObject);

begin
  HandlePartialKey;
  end;


procedure TSelectForm.scbFakeScroll(Sender: TObject;
     ScrollCode: TScrollCode; var ScrollPos: integer);

var
  ratio: real;

begin
  case ScrollCode of
    scTop: Top;
    scBottom: Bottom;
    scLineUp:  Up1;
    scLineDown: Down1;
    scPageUp: PageUp;
    scPageDown: PageDown;
    scPosition:  begin
      ratio := ScrollPos/scbFake.Max;
      GoToThumbPosition(ratio);
      end;
    end;
  ActiveControl := edtPartial;
  end;


procedure TSelectForm.FormKeyDown(Sender: TObject; var Key: word;
      Shift: TShiftState);

begin
  case Key of
    VK_TAB:    TabPress;
    VK_RETURN: OKClick(Sender);
    VK_ESCAPE: CancelClick(Sender);
    VK_PRIOR:  PageUp;
    VK_NEXT:   PageDown;
    VK_END:    Bottom;
    VK_HOME:   Top;
    VK_UP:     Up1;
    VK_DOWN:   Down1;
    end;
  if Key < 30
  then Key := 0;
  ActiveControl := edtPartial;
  end;


procedure TSelectForm.FormKeyUp(Sender: TObject; var Key: word;
      Shift: TShiftState);

begin
  if Key = VK_TAB
  then TabPress;
  if Key < 30
  then Key := 0;
  ActiveControl := edtPartial;
  end;


procedure TSelectForm.TabPress;

var
  w,j: integer;

begin
  {see if there is a highlighted item}
  j := -1;
  for w := 1 to Actual do
    if clbChoose.Selected[w-1] then j := w-1;
  if j = -1
  then exit;
  with clbChoose do begin
    if FSetUp.OneOnly
    then begin
      lbxShow.Items.Clear;
      lbxShow.Items.Add(Items[j]);
      end
    else if lbxShow.Items.IndexOf(Items[j]) = -1
    then lbxShow.Items.Add(Items[j]);
    end;
  PutInChecks;
  end;


{===========================================================}
{    Part 3     Speed Button routines                       }
{===========================================================}


procedure TSelectForm.OkClick(Sender: TObject);

begin
  if FSetUp.OneOnly
    then if lbxShow.Items.Count > 0
         then FSetUp.Chosen := lbxShow.Items[0]
         else FSetUp.Chosen := ''
  else ReadSelections;
  Close;
  ModalResult := mrOK;
  end;


procedure TSelectForm.CancelClick(Sender: TObject);

begin
  if FSetUp.OneOnly
  then FSetUp.Chosen := '';
  Close;
  ModalResult := mrCancel;
  end;


procedure TSelectForm.SyncClick(Sender: TObject);

begin
  CheckString := edtPartial.Text;
  HandlePartialKey;
  end;



procedure TSelectForm.InsertClick(Sender: TObject);

var
  found: shortstring;

begin
  with FSetUp do begin
    if edtPartial.Text = ''
    then begin
      ShowMessage('The null string is not a valid key.');
      exit;
      end;
    found := TheKey.FindKey(edtPartial.Text);
    if edtPartial.Text = found
    then begin
      ShowMessage('The key '+found+' already exists');
      exit;
      end;
    TheKey.InsertKey(edtPartial.Text);
    NewList.Add(edtPartial.Text);
    if FSetUp.OneOnly
    then begin
      lbxShow.Items.Clear;
      lbxShow.Items.Add(edtPartial.Text);
      end
    else if lbxShow.Items.IndexOf(edtPartial.Text) = -1
         then lbxShow.Items.Add(edtPartial.Text);
    GoToKey(edtPartial.Text);
    ShowScroll := Actual < TheKey.ActiveKeys;
    scbFAke.Visible := ShowScroll;
    ActiveControl := edtPartial;
    end;
  end;


{===========================================================}
{     Part 4       Drag and drop stuff                      }
{===========================================================}

procedure TSelectForm.clbChooseMouseDown(Sender: TObject;
    Button: TMouseButton; Shift: TShiftState; X,Y: integer);

var
  index: integer;

begin
  if Button = mbLeft
  then with Sender as TCheckListBox do begin
    index := ItemAtPos(Point(X,Y),TRUE);
    if index >= 0
    then begin
      BeginDrag(FALSE);
      Dragger := Items[index];
      end;
    end;
  end;


procedure TSelectForm.clbChooseDragOver(Sender, Source: TObject;
     X,Y: integer; State: TDragState; var Accept: boolean);

begin
  if Source is TListBox
  then Accept := TRUE;
  end;


procedure TSelectForm.clbChooseDragDrop(Sender,Source: TObject;
     X,Y: integer);

var
  index: integer;

begin
  if Source is TListBox
  then begin
    index := lbxShow.Items.IndexOf(Dragger);
    if index <> -1
    then lbxShow.Items.Delete(index);
    PutInChecks;
    end;
  end;


procedure TSelectForm.lbxShowMouseDown(Sender: TObject;
     Button: TMouseButton; Shift: TShiftState; X,Y: integer);

var
  index: integer;

begin
  if Button = mbLeft
  then with Sender as TListBox do begin
    index := ItemAtPos(Point(X,Y), TRUE);
    if index >=0
    then begin
      BeginDrag(FALSE);
      Dragger := Items[index];
      end;
    end;
  end;


procedure TSelectForm.lbxShowDragOver(Sender, Source: TObject;
     X,Y: integer; State: TDragState; var Accept: boolean);


begin
  if Source is TCheckListBox
  then Accept := TRUE;
  end;




procedure TSelectForm.lbxShowDragDrop(Sender,Source: TObject;
     X,Y: integer);


begin
  if Source is TCheckListBox
  then if FSetUp.OneOnly
       then begin
         lbxShow.Items.Clear;
         lbxShow.Items.Add(Dragger);
         end
       else if lbxShow.Items.IndexOf(Dragger) = -1
            then lbxShow.Items.Add(Dragger);
  PutInChecks;
  end;


{===========================================================}
{     Part 5       Initialization                           }
{===========================================================}


procedure TSelectForm.CalculateActual;

begin
  if FSetUp.TheKey.ActiveKeys < 10
  then Actual := FSetUp.TheKey.ActiveKeys
  else if FSetUp.TheKey.ActiveKeys < 100
       then Actual := 10
       else Actual := 20;
  if Actual > FSetUp.TheKey.ActiveKeys
  then Actual := FSetUp.TheKey.ActiveKeys;
  if Actual = 0
  then Actual := 1;
  ShowScroll := (Actual < FSetUp.TheKey.ActiveKeys);
  end;



procedure TSelectForm.ResizeComponents(Sender: TObject);

var
  x,NK: word;
  mid: shortstring;
  ScrollInfo: TScrollInfo;


begin
  with Self do begin
    if ClientHeight < MinimumHeight
    then ClientHeight := MinimumHeight;
    if clbChoose.Items.Count > 0
    then mid := clbChoose.Items[Actual div 2];
    {We cannot resize for more than ActiveKeys}
    NK := (Self.ClientHeight-70-EditHeight) div IHeight-1;
    if not FSetUp.OneOnly
    then dec(NK,4);
    if NK > FSetUp.TheKey.ActiveKeys
    then NK := FSetUp.TheKey.ActiveKeys;
    Actual := NK;
    if Actual < 1
    then Actual := 1;

    pnlFrame.Width := Self.ClientWidth-14;
    pnlFrame.Height := Self.ClientHeight-14;
    edtPartial.Width := pnlFrame.Width-55;
    if FSetUp.AllowInserts
    then x := (pnlFrame.Height - 100) div 5
    else x := (pnlFrame.Height - 75) div 4;
    if FSetUp.AllowInserts
    then begin
      sbnInsert.Left := pnlFrame.Width-38;
      sbnInsert.Top := x;
      end;
    sbnSync.Left := pnlFrame.Width-38;
    if FSetUp.AllowInserts
    then sbnSync.Top := 2*x+25
    else sbnSync.Top := x;
    sbnCancel.Left := pnlFrame.Width-38;
    if FSetUp.AllowInserts
    then sbnCancel.Top := 3*x+50
    else sbnCancel.Top := 2*x+25;
    sbnOK.Left := pnlFrame.Width-38;
    if FSetUp.AllowInserts
    then sbnOK.Top := 4*x+75
    else sbnOK.Top := 3*x+50;
    lbxShow.Width := pnlFrame.Width-55;
    clbChoose.Width := pnlFrame.Width-55;
    clbChoose.Height := 4 + NK*IHeight;
    scbFake.Height := clbChoose.Height - 4;
    scbFake.Left := clbChoose.Width - 4;
    ScrollInfo.cbSize := sizeof(ScrollInfo);
    ScrollInfo.fMask := SIF_PAGE;
    ScrollInfo.nPage  := Actual;
    SetScrollInfo(scbFake.Handle,SB_CTL,ScrollInfo,TRUE);
    scbFake.Visible := FSetUp.TheKey.ActiveKeys > Actual;
    scbFake.Min := 0;
    scbFake.Max := FSetUp.TheKey.ActiveKeys;
    scbFake.SmallChange := 1;
    scbFake.LargeChange := Actual;
    end;
  if Actual > 2
  then GoToKey(mid);
  end;



constructor TSelectForm.Init(var SetUp: TSelectSetUp);

var
  KWidth: word;
  ScrollInfo: TScrollInfo;
  f: shortstring;
  j,x: word;
  switch: boolean;

begin
  CreateNew(Application);
  with Self do begin
    {Copy parameters}
    FSetUp := SetUp;
    if FSetUp.KeyList <> nil
    then begin
      FSetUp.KeyList.Duplicates := dupIgnore;
      FSetUp.KeyList.Sorted := TRUE;
      end;
    IHeight := Canvas.TextHeight('Wy')+3;
    if FSetUp.TheCharCase = ecLowerCase
    then f := 'w'
    else f := 'W';
    for j := 2 to FSetUp.TheKey.KeyLength do
      if FSetUp.TheCharCase = ecUpperCase
      then f := f + 'W'
      else f := f + 'w';
    KWidth := Canvas.TextWidth(f) * 85 div 100;

    edtPartial := TEdit.Create(Self);
    with edtPartial do begin
      Parent := Self;
      AutoSize := TRUE;
      EditHeight := Height;
      Free;
      end;

    {Calculate Actual}
    CalculateActual;

    if FSetUp.OneOnly
    then ClientHeight := 70 + EditHeight +
         IHeight*(Actual + 1)
    else ClientHeight := 70 + EditHeight +
         IHeight*(Actual + 5);
    ClientWidth := 75 + KWidth;
    MinimumHeight := 100 + EditHeight + 6*IHeight;

    Position := poScreenCenter;
    Caption := FSetUp.Header;
    if FSetUp.TheKey.ActiveKeys > 100
    then BorderStyle := bsSizeable
    else BorderStyle := bsDialog;
    BorderIcons := [biSystemMenu];
    KeyPreview := TRUE;
    OnKeyDown := FormKeyDown;
    OnKeyUp := FormKeyUp;

    pnlFrame := TPanel.Create(Self);
    with pnlFrame do begin
      Parent :=Self;
      Left := 5;
      Top := 5;
      Width := Self.ClientWidth-14;
      Height := Self.ClientHeight - 14;
      BevelInner := bvLowered;
      BevelOuter := bvRaised;
      BevelWidth := 2;
      end;

    edtPartial := TEdit.Create(Self);
    with edtPartial do begin
      Parent := pnlFrame;
      CharCase := FSetUp.TheCharCase;
      Left := 10;
      Top := 10;
      Height := EditHeight;
      Width := pnlFrame.Width - 55;
      AutoSize := TRUE;
      Hint := 'Enter partial key';
      ShowHint := TRUE;
      OnChange := edtPartialChange;
      end;

    if FSetUp.AllowInserts
    then x := (pnlFrame.Height - 100) div 5
    else x := (pnlFrame.Height - 75) div 4;

    if FSetUp.AllowInserts
    then begin
      sbnInsert := TSpeedButton.Create(Self);
      with sbnInsert do begin
        Parent := pnlFrame;
        Left := pnlFrame.Width - 38;
        Top := x;
        Width := 29;
        Height := 29;
        Hint := 'Insert item';
        AllowAllUp := TRUE;
        Glyph.LoadFromResourceName(HInstance,'ADD');
        ParentShowHint := FALSE;
        ShowHint := TRUE;
        OnClick := InsertClick;
        end;
      end;

    sbnSync := TSpeedButton.Create(Self);
    with sbnSync do begin
      Parent := pnlFrame;
      Left := pnlFrame.Width - 38;
      if FSetUp.AllowInserts
      then Top := 2*x + 25
      else Top := x;
      Width := 29;
      Height := 29;
      Hint := 'Synchronize';
      AllowAllUp := TRUE;
      Glyph.LoadFromResourceName(HInstance,'SYNC');
      ParentShowHint := FALSE;
      ShowHint := TRUE;
      OnClick := SyncClick;
      end;

    sbnCancel := TSpeedButton.Create(Self);
    with sbnCancel do begin
      Parent := pnlFrame;
      Left := pnlFrame.Width - 38;
      if FSetUp.AllowInserts
      then Top := 3*x + 50
      else Top := 2*x + 25;
      Width := 29;
      Height := 29;
      Hint := 'Cancel selections';
      AllowAllUp := TRUE;
      Glyph.LoadFromResourceName(HInstance,'CANCEL');
      ParentShowHint := FALSE;
      ShowHint := TRUE;
      OnClick := CancelClick;
      end;

    sbnOK := TSpeedButton.Create(Self);
    with sbnOK do begin
      Parent := pnlFrame;
      Left := pnlFrame.Width - 38;
      if FSetUp.AllowInserts
      then Top := 4*x + 75
      else Top := 3*x + 50;
      Width := 29;
      Height := 29;
      Hint := 'Accept selections';
      AllowAllUp := TRUE;
      Glyph.LoadFromResourceName(HInstance,'POST');
      ParentShowHint := FALSE;
      ShowHint := TRUE;
      OnClick := OkClick;
      end;

    lbxShow := TListBox.Create(Self);
    with lbxShow do begin
      Parent := pnlFrame;
      Left := 10;
      Top := 20 + EditHeight;
      Width := pnlFrame.Width - 55;
      if FSetUp.OneOnly
      then Height := 4 + IHeight
      else Height := 4 + 5*IHeight;
      Style := lbOwnerDrawFixed;
      IntegralHeight := TRUE;
      ItemHeight := IHeight;
      Ctl3d := TRUE;
      MultiSelect := FALSE;
      Sorted := TRUE;
      Color := FSetUp.TheContrast;
      if FSetUp.OneOnly
      then Hint := 'Choice made as of now'
      else Hint := 'Choices made as of now';
      ShowHint := TRUE;
      OnDblClick := lbxShowDblClick;
      OnMouseDown := lbxShowMouseDown;
      OnDragOver := lbxShowDragOver;
      OnDragDrop := lbxShowDragDrop;
      if not FSetUp.OneOnly
      then Items.Assign(FSetUp.KeyList);
      end;

      clbChoose := TCheckListBox.Create(Self);
      with clbChoose do begin
        Parent := pnlFrame;
        Style := lbOwnerDrawFixed;
        IntegralHeight := TRUE;
        ItemHeight := IHeight;
        Left := 10;
        Top := 10 +lbxShow.Top + lbxShow.Height;
        Width := pnlFrame.Width - 55;
        Height := 4 +IHeight*Actual;
        AllowGrayed := FALSE;
        Ctl3d := TRUE;
        Sorted := TRUE;
        Hint := 'Choose from these items';
        ShowHint := TRUE;
        OnClickCheck := clbChooseClickCheck;
        OnDblClick := clbChooseDblClick;
        OnMouseDown := clbChooseMouseDown;
        OnDragOver := clbChooseDragOver;
        OnDragDrop := clbChooseDragDrop;
        end;


    scbFake := TScrollBar.Create(Self);
    with scbFake do begin
      Parent := pnlFrame;
      Kind := sbVertical;
      Top := clbChoose.Top +1;
      Left := clbChoose.Width - 4;
      Width := 13;
      Height := clbChoose.Height - 4;
      ScrollInfo.cbSize := sizeof(ScrollInfo);
      ScrollInfo.fMask := SIF_PAGE;
      ScrollInfo.nPage  := Actual;
      SetScrollInfo(scbFake.Handle,SB_CTL,ScrollInfo,TRUE);
      Visible := ShowScroll;
      Min := 0;
      Max := FSetUp.TheKey.ActiveKeys;
      SmallChange := 1;
      LargeChange := Actual;
      OnScroll := scbFakeScroll;
      end;

    ActiveControl := edtPartial;
    lbxShow.Clear;
    lbxShow.ItemIndex := -1;
    f := '';
    if FSetUp.OneOnly
    then if FSetUp.Chosen <> ''
         then begin
           f := FSetUp.Chosen;
           lbxShow.Items.Add(f);
           lbxShow.ItemIndex := lbxShow.Items.IndexOf(f);
           end
         else
    else if FSetUp.KeyList.Count <> 0
         then begin
           lbxShow.Items.Assign(FSetUp.KeyList);
           lbxShow.ItemIndex := -1;
           end;
    end;
  OnResize := ResizeComponents;
  if FSetUp.TheKey.ActiveKeys > 0
  then GoToKey(f);
  end;


   end.



{=======================END OF Select MODULE =================}



